package es.verdnatura.presentation.view.feature.calidad.fragment

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import es.verdnatura.domain.SilexCallback
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.calidad.model.ItemBuyerListVO
import es.verdnatura.presentation.view.feature.calidad.model.ItemBuyerVO
import retrofit2.Response

class QaualityViewModel(val context: Context) : BaseViewModel(context) {

    //private val getQualityUserCase: GetQualityUserCase = GetQualityUserCase(context)

    private val _buyersList by lazy { MutableLiveData<ItemBuyerListVO>() }
    val buyersList: LiveData<ItemBuyerListVO>
        get() = _buyersList


    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response


    fun itemShelvingBuyerTask(userFk: String) {
        silex.itemShelvingBuyerTask(userFk).enqueue(object :
            SilexCallback<List<ItemBuyerVO>>(context) {
            override fun onError(t: Throwable) {
                val listError: ArrayList<ItemBuyerVO> = ArrayList()
                listError.add(
                    ItemBuyerVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                )
                _buyersList.value = ItemBuyerListVO(listError)
            }

            override fun onSuccess(response: Response<List<ItemBuyerVO>>) {
                super.onSuccess(response)
                if (response.body() != null) {
                    _buyersList.value = response.body()?.let { ItemBuyerListVO(it) }
                } else {
                    val listError: ArrayList<ItemBuyerVO> = ArrayList()
                    listError.add(
                        ItemBuyerVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    )
                    _buyersList.value = ItemBuyerListVO(listError)
                }
            }

        })
    }

    fun itemShelvingStarsUpdate(vId: String, vStars: String) {
        silex.itemShelvingStarsUpdate(vId, vStars).enqueue(object : SilexCallback<Any>(context) {
            override fun onError(t: Throwable) {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                )
            }

            override fun onSuccess(response: Response<Any>) {
                _response.value = ResponseItemVO(isError = false, response = response.message())
            }
        })
    }
}