package es.verdnatura.presentation.view.feature.faltas.adapter

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import es.verdnatura.R
import es.verdnatura.databinding.ItemFaltasRowBinding
import es.verdnatura.presentation.common.OnFaltasNichoClickListener
import es.verdnatura.presentation.common.OnFaltasReviewClickListener
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.view.feature.faltas.model.ItemFaltasVO
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO

class FaltasAdapter (
    private val items: List<ItemFaltasVO>,
    private val onFaltasNichoClickListener: OnFaltasNichoClickListener,
    private val onFaltasReviewClickListener:OnFaltasReviewClickListener,
    private val onPasillerosItemClickListener: OnPasillerosItemClickListener
): RecyclerView.Adapter<FaltasAdapter.ItemHolder> () {


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ItemHolder {
        return ItemHolder(
            ItemFaltasRowBinding.inflate(LayoutInflater.from(parent.context),parent,false)
        )
    }

    override fun getItemCount() =items.size

    override fun onBindViewHolder(holder: ItemHolder, position: Int) {
        holder.bind(items[position])
    }

    inner class ItemHolder(
        val binding: ItemFaltasRowBinding
    ) : RecyclerView.ViewHolder(binding.root){
        private val res = binding.root.context.resources
        fun bind(item: ItemFaltasVO) {
            binding.apply {
                this.item = item
                if(item.producer.isNullOrEmpty()){
                    itemProducer.visibility = View.GONE
                }else{
                    itemProducer.visibility = View.VISIBLE
                }
                itemNicho.setOnClickListener {
                    onFaltasNichoClickListener.onFaltasNichoClickListener(item)
                }
                itemFaltas.setOnClickListener {
                    onFaltasReviewClickListener.onFaltasReviewClickListener(item)
                }
                itemFk.setOnClickListener {
                    onPasillerosItemClickListener.onPasillerosItemClickListener(PasillerosItemVO(title = res.getString(
                        R.string.titleItemConsult)),item.itemFk)
                }
                itemUp.setOnClickListener {
                    onPasillerosItemClickListener.onPasillerosItemClickListener(PasillerosItemVO(title = res.getString(R.string.titleItemSearch)),item.itemFk)
                }
                itemDown.setOnClickListener {
                    onPasillerosItemClickListener.onPasillerosItemClickListener(PasillerosItemVO(title = res.getString(R.string.titleItemSearch)),item.itemFk)
                }
            }
        }
    }
}