package es.verdnatura.presentation.view.feature.faltas.fragment

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import es.verdnatura.domain.SilexCallback
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.faltas.model.ItemFaltasListVO
import es.verdnatura.presentation.view.feature.faltas.model.ItemFaltasVO
import retrofit2.Call
import retrofit2.Response

class FaltasViewModel(val context: Context) : BaseViewModel(context) {

    private val _faltasList by lazy { MutableLiveData<ItemFaltasListVO>() }
    val faltasList: LiveData<ItemFaltasListVO>
        get() = _faltasList

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response


    //val loadFaltasList = Transformations.map(_faltasList) { Event(it) }


    fun getFaltas(user: String, password: String, warehouseFk: String) {
        silex.faultsReview(user, password, warehouseFk).enqueue(object :
            SilexCallback<List<ItemFaltasVO>>(context) {
            override fun onError(t: Throwable) {
                val listError: ArrayList<ItemFaltasVO> = ArrayList()
                listError.add(
                    ItemFaltasVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                )
                _faltasList.value = ItemFaltasListVO(listError)
            }

            override fun onSuccess(response: Response<List<ItemFaltasVO>>) {
                if (response.body() != null) {
                    _faltasList.value = response.body()?.let { ItemFaltasListVO(it) }
                } else {
                    val listError: ArrayList<ItemFaltasVO> = ArrayList()
                    listError.add(
                        ItemFaltasVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    )
                    _faltasList.value = ItemFaltasListVO(listError)
                }
            }

        })
    }

    fun itemStockUpdate(
        itemFk: String,
        warehouseFk: String,
        newValue: String,
        isTrash: String
    ) {
        silex.itemTrash(itemFk, warehouseFk, newValue, isTrash)
            .enqueue(object :
                SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onResponse(call: Call<Any>, response: Response<Any>) {
                    if (!response.isSuccessful) {
                        _response.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    } else {
                        _response.value =
                            ResponseItemVO(isError = false, response = response.message())
                    }

                }

            })
    }

    fun faultsReview_isChecked(
        user: String,
        password: String,
        itemFk: String,
        warehouseFk: String
    ) {
        silex.faultsReview_isChecked(user, password, itemFk, warehouseFk)
            .enqueue(object :
                SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {

                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {

                    _response.value =
                        ResponseItemVO(isError = false, response = response.message())

                }
            })
    }

}
