package es.verdnatura.presentation.view.feature.paletizador.fragment


import android.content.Context
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import es.verdnatura.domain.SilexCallback
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionTruckList
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionTruckVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class ExpeditionTruckListViewModel(val context: Context) : BaseViewModel(context) {

    private val _expeditionTruckList by lazy { MutableLiveData<ItemExpeditionTruckList>() }
    val loadExpeditionTruckList = Transformations.map(_expeditionTruckList) { Event(it) }

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val loadResponseExpeditionAdd = Transformations.map(_response) { Event(it) }


    fun expeditionTruckList() {
        silex.expeditionTruckList().enqueue(object :
            SilexCallback<List<ItemExpeditionTruckVO>>(context) {
            override fun onError(t: Throwable) {
                val listError: ArrayList<ItemExpeditionTruckVO> = ArrayList()
                listError.add(
                    ItemExpeditionTruckVO(
                        0,
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                )
                _expeditionTruckList.value = ItemExpeditionTruckList(listError)
            }

            override fun onSuccess(response: Response<List<ItemExpeditionTruckVO>>) {
                if (response.body() != null) {
                    _expeditionTruckList.value =
                        response.body()?.let { ItemExpeditionTruckList(it) }
                } else {
                    val listError: ArrayList<ItemExpeditionTruckVO> = ArrayList()
                    listError.add(
                        ItemExpeditionTruckVO(
                            0,
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    )
                    _expeditionTruckList.value = ItemExpeditionTruckList(listError)
                }
            }

        })
    }

    fun expeditionTruckAdd(vHour: String, vDescription: String) {
        silex.expeditionTruckAdd(vHour, vDescription).enqueue(object : Callback<Any> {
            override fun onFailure(call: Call<Any>, t: Throwable) {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                )
            }

            override fun onResponse(call: Call<Any>, response: Response<Any>) {
                if (!response.isSuccessful) {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(
                            nameofFunction(this),
                            response.message()
                        )
                    )
                } else {
                    _response.value = ResponseItemVO(isError = false, response = response.message())
                }

            }

        })
    }

}
