package es.verdnatura.presentation.view.feature.reposicion.fragment

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import es.verdnatura.domain.SilexCallback
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.presacador.model.PreSacadorItemVO
import es.verdnatura.presentation.view.feature.sacador.model.PlacementSupplyListVO
import es.verdnatura.presentation.view.feature.sacador.model.PlacementSupplyVO
import retrofit2.Response

class ReposicionViewModel(val context: Context) : BaseViewModel(context) {


    private val _salesList by lazy { MutableLiveData<List<PreSacadorItemVO>>() }
    val salesList: LiveData<List<PreSacadorItemVO>>
        get() = _salesList

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response

    private val _responseClose by lazy { MutableLiveData<ResponseItemVO>() }
    val responseClose: LiveData<ResponseItemVO>
        get() = _responseClose

    private val _placementSuppleyList by lazy { MutableLiveData<PlacementSupplyListVO>() }
    val placementSuppleyList: LiveData<PlacementSupplyListVO>
        get() = _placementSuppleyList

    private val _responseCode by lazy { MutableLiveData<ResponseItemVO>() }
    val responseCode: LiveData<ResponseItemVO>
        get() = _responseCode

    fun itemPlacementSupplyGetOrder(
        sectorFk: Int

    ) {
        silex.itemPlacementSupplyGetOrder(sectorFk)
            .enqueue(object :
                SilexCallback<List<PreSacadorItemVO>>(context) {
                override fun onError(t: Throwable) {
                    _salesList.value = listOf()
                }

                override fun onSuccess(response: Response<List<PreSacadorItemVO>>) {
                    if (response.body() != null) {
                        _salesList.value = response.body()?.let { it }
                    } else {
                        _salesList.value = listOf()
                    }
                }
            })
    }

    fun parking(
        ticketFk: String,
        parking: String

    ) {
        silex.shelvingPark(ticketFk, parking)
            .enqueue(object : SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                    _response.value =
                        ResponseItemVO(isError = false, response = response.message())
                }
            })
    }

    fun itemPlacementSupplyAiming(
        shelvingFk: String,
        quantity: Int,
        itemFk: Int

    ) {
        silex.itemPlacementSupplyAiming(
            shelvingFk,
            quantity,
            itemFk
        ).enqueue(object :
            SilexCallback<List<PlacementSupplyVO>>(context) {
            override fun onError(t: Throwable) {
                val listError: ArrayList<PlacementSupplyVO> = ArrayList()
                listError.add(
                    PlacementSupplyVO(
                        itemShelvingFk = 0,
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                )
                _placementSuppleyList.value = PlacementSupplyListVO(listError)
            }

            override fun onSuccess(response: Response<List<PlacementSupplyVO>>) {
                if (response.body() != null) {
                    _placementSuppleyList.value = response.body()?.let { PlacementSupplyListVO(it) }
                } else {
                    val listError: ArrayList<PlacementSupplyVO> = ArrayList()
                    listError.add(
                        PlacementSupplyVO(
                            itemShelvingFk = 0,
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    )
                    _placementSuppleyList.value = PlacementSupplyListVO(listError)
                }
            }
        })
    }

    fun getIdFromCode(usuario: String, password: String, code: String) {
        silex.getIdFromCode(usuario, password, code)
            .enqueue(object : SilexCallback<String>(context) {
                override fun onError(t: Throwable) {
                    _responseCode.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<String>) {
                    //sergio:espera item en el body
                    if (response.body() == null) {
                        _responseCode.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    } else {
                        _responseCode.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }


    fun itemShelvingPlacementSupplyAdd(
        itemShelvingFk: String,
        itemPlacementSupplyFk: String,
        quantity: String

    ) {
        silex.itemShelvingPlacementSupplyAdd(
            itemShelvingFk,
            itemPlacementSupplyFk,
            quantity
        ).enqueue(object : SilexCallback<Any>(context) {
            override fun onError(t: Throwable) {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                )
            }

            override fun onSuccess(response: Response<Any>) {
                _response.value = ResponseItemVO(isError = false, response = response.message())
            }
        })
    }

    fun itemPlacementSupplyCloseOrder(
        id: Int,
        quantity: Int
    ) {
        silex.itemPlacementSupplyCloseOrder(id, quantity)
            .enqueue(object : SilexCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _responseClose.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Any>) {
                    _responseClose.value =
                        ResponseItemVO(isError = false, response = response.message())
                }
            })
    }

}